# frozen_string_literal: true

require 'cloudscreener_ruby_sdk/api'
require 'cloudscreener_ruby_sdk/virtual_machine'

RSpec.describe Cloudscreener::Api do
  context 'with errors' do
    it 'should raise an error when credentials arn\'t set' do
      api = Cloudscreener::Api.new

      expect { api.search }.to raise_exception('Credentials not set, please use configure_credentials().')
    end

    it 'should raise an error when no VM is added' do
      api = Cloudscreener::Api.new('a', 'b')

      expect { api.search }.to raise_exception('You must select at least one VM. Please use add_vm()')
    end

    it 'should raise an error when no providers is added' do
      api = Cloudscreener::Api.new('a', 'b')
      api.add_vm(Cloudscreener::VirtualMachine.new)

      expect { api.search }.to \
        raise_exception('You must select at least one provider. Please use api.providers = %w[...]')
    end
  end

  context 'query computation' do
    it 'should compute a simple query' do
      backend = Cloudscreener::VirtualMachine.new('backend')
      backend.cpus = 2
      backend.os = 'rhel'

      api = Cloudscreener::Api.new
      api.fields = %i[name cpus]
      api.providers = %i[aws]
      api.add_vm(backend)

      expectation = {
        multi: [{
          match: [{ field: 'pslg', op: 'in', value: %i[aws] }, { field: 'os', value: 'rhel' }],
          near: [{ field: 'cpus', value: 2 }],
          qty: 1
        }],
        slice: %i[name cpus],
        currency: 'USD'
      }

      expect(api.send('compute_query')).to eq(expectation)
    end

    it 'should compute complex query' do
      backend = Cloudscreener::VirtualMachine.new('backend')
      backend.cpus = 2
      backend.os = 'rhel'

      frontend = Cloudscreener::VirtualMachine.new('frontend')
      frontend.ram = 8
      frontend.os = 'linux'
      frontend.qty = 4

      api = Cloudscreener::Api.new
      api.fields = %i[name cpus dcnas]
      api.providers = %i[aws gcp]
      api.currency = 'EUR'
      api.add_vm(backend)
      api.add_vm(frontend)

      expectation = {
          multi: [
            {
              match: [{ field: 'pslg', op: 'in', value: %i[aws gcp] }, { field: 'os', value: 'rhel' }],
              near: [{ field: 'cpus', value: 2 }],
              qty: 1
            },{
              match: [{ field: 'pslg', op: 'in', value: %i[aws gcp] }, { field: 'os', value: 'linux' }],
              near: [{ field: 'ram', value: 8 }],
              qty: 4
            }
          ],
          slice: %i[name cpus dcnas],
          currency: 'EUR'
      }

      expect(api.send('compute_query')).to eq(expectation)
    end
  end
end
